from typing import Literal, Set, cast

MarketIfTouchedOrderTransactionReason = Literal["CLIENT_ORDER", "REPLACEMENT"]

MARKET_IF_TOUCHED_ORDER_TRANSACTION_REASON_VALUES: Set[
    MarketIfTouchedOrderTransactionReason
] = {
    "CLIENT_ORDER",
    "REPLACEMENT",
}


def check_market_if_touched_order_transaction_reason(
    value: str,
) -> MarketIfTouchedOrderTransactionReason:
    if value in MARKET_IF_TOUCHED_ORDER_TRANSACTION_REASON_VALUES:
        return cast(MarketIfTouchedOrderTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_IF_TOUCHED_ORDER_TRANSACTION_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.price_bucket import PriceBucket


T = TypeVar("T", bound="Price")


@_attrs_define
class Price:
    """The Price representation

    Attributes:
        instrument (Union[Unset, str]): The Price's Instrument.
        tradeable (Union[Unset, bool]): Flag indicating if the Price is tradeable or not
        timestamp (Union[Unset, str]): The date/time when the Price was created.
        base_bid (Union[Unset, str]): The base bid price as calculated by pricing.
        base_ask (Union[Unset, str]): The base ask price as calculated by pricing.
        bids (Union[Unset, List['PriceBucket']]): The list of prices and liquidity available on the Instrument's bid
            side. It is possible for this list to be empty if there is no bid liquidity currently available for the
            Instrument in the Account.
        asks (Union[Unset, List['PriceBucket']]): The list of prices and liquidity available on the Instrument's ask
            side. It is possible for this list to be empty if there is no ask liquidity currently available for the
            Instrument in the Account.
        closeout_bid (Union[Unset, str]): The closeout bid price. This price is used when a bid is required to closeout
            a Position (margin closeout or manual) yet there is no bid liquidity. The closeout bid is never used to open a
            new position.
        closeout_ask (Union[Unset, str]): The closeout ask price. This price is used when an ask is required to closeout
            a Position (margin closeout or manual) yet there is no ask liquidity. The closeout ask is never used to open a
            new position.
    """

    instrument: Union[Unset, str] = UNSET
    tradeable: Union[Unset, bool] = UNSET
    timestamp: Union[Unset, str] = UNSET
    base_bid: Union[Unset, str] = UNSET
    base_ask: Union[Unset, str] = UNSET
    bids: Union[Unset, List["PriceBucket"]] = UNSET
    asks: Union[Unset, List["PriceBucket"]] = UNSET
    closeout_bid: Union[Unset, str] = UNSET
    closeout_ask: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        instrument = self.instrument

        tradeable = self.tradeable

        timestamp = self.timestamp

        base_bid = self.base_bid

        base_ask = self.base_ask

        bids: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bids, Unset):
            bids = []
            for bids_item_data in self.bids:
                bids_item = bids_item_data.to_dict()
                bids.append(bids_item)

        asks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.asks, Unset):
            asks = []
            for asks_item_data in self.asks:
                asks_item = asks_item_data.to_dict()
                asks.append(asks_item)

        closeout_bid = self.closeout_bid

        closeout_ask = self.closeout_ask

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if tradeable is not UNSET:
            field_dict["tradeable"] = tradeable
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if base_bid is not UNSET:
            field_dict["baseBid"] = base_bid
        if base_ask is not UNSET:
            field_dict["baseAsk"] = base_ask
        if bids is not UNSET:
            field_dict["bids"] = bids
        if asks is not UNSET:
            field_dict["asks"] = asks
        if closeout_bid is not UNSET:
            field_dict["closeoutBid"] = closeout_bid
        if closeout_ask is not UNSET:
            field_dict["closeoutAsk"] = closeout_ask

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.price_bucket import PriceBucket

        d = src_dict.copy()
        instrument = d.pop("instrument", UNSET)

        tradeable = d.pop("tradeable", UNSET)

        timestamp = d.pop("timestamp", UNSET)

        base_bid = d.pop("baseBid", UNSET)

        base_ask = d.pop("baseAsk", UNSET)

        bids = []
        _bids = d.pop("bids", UNSET)
        for bids_item_data in _bids or []:
            bids_item = PriceBucket.from_dict(bids_item_data)

            bids.append(bids_item)

        asks = []
        _asks = d.pop("asks", UNSET)
        for asks_item_data in _asks or []:
            asks_item = PriceBucket.from_dict(asks_item_data)

            asks.append(asks_item)

        closeout_bid = d.pop("closeoutBid", UNSET)

        closeout_ask = d.pop("closeoutAsk", UNSET)

        price = cls(
            instrument=instrument,
            tradeable=tradeable,
            timestamp=timestamp,
            base_bid=base_bid,
            base_ask=base_ask,
            bids=bids,
            asks=asks,
            closeout_bid=closeout_bid,
            closeout_ask=closeout_ask,
        )

        price.additional_properties = d
        return price

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.order_request import OrderRequest


T = TypeVar("T", bound="CreateOrderBody")


@_attrs_define
class CreateOrderBody:
    """
    Attributes:
        order (Union[Unset, OrderRequest]): The base Order specification used when requesting that an Order be created.
            Each specific Order-type extends this definition.
    """

    order: Union[Unset, "OrderRequest"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        order: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order, Unset):
            order = self.order.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if order is not UNSET:
            field_dict["order"] = order

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.order_request import OrderRequest

        d = src_dict.copy()
        _order = d.pop("order", UNSET)
        order: Union[Unset, OrderRequest]
        if isinstance(_order, Unset):
            order = UNSET
        else:
            order = OrderRequest.from_dict(_order)

        create_order_body = cls(
            order=order,
        )

        create_order_body.additional_properties = d
        return create_order_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.market_order_request_position_fill import (
    check_market_order_request_position_fill,
)
from ..models.market_order_request_position_fill import MarketOrderRequestPositionFill
from ..models.market_order_request_time_in_force import (
    check_market_order_request_time_in_force,
)
from ..models.market_order_request_time_in_force import MarketOrderRequestTimeInForce
from ..models.market_order_request_type import check_market_order_request_type
from ..models.market_order_request_type import MarketOrderRequestType
from typing import Union

if TYPE_CHECKING:
    from ..models.trailing_stop_loss_details import TrailingStopLossDetails
    from ..models.take_profit_details import TakeProfitDetails
    from ..models.stop_loss_details import StopLossDetails
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="MarketOrderRequest")


@_attrs_define
class MarketOrderRequest:
    """A MarketOrderRequest specifies the parameters that may be set when creating a Market Order.

    Attributes:
        type (Union[Unset, MarketOrderRequestType]): The type of the Order to Create. Must be set to "MARKET" when
            creating a Market Order.
        instrument (Union[Unset, str]): The Market Order's Instrument.
        units (Union[Unset, str]): The quantity requested to be filled by the Market Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        time_in_force (Union[Unset, MarketOrderRequestTimeInForce]): The time-in-force requested for the Market Order.
            Restricted to FOK or IOC for a MarketOrder.
        price_bound (Union[Unset, str]): The worst price that the client is willing to have the Market Order filled at.
        position_fill (Union[Unset, MarketOrderRequestPositionFill]): Specification of how Positions in the Account are
            modified when the Order is filled.
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        take_profit_on_fill (Union[Unset, TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Union[Unset, StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Union[Unset, TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
    """

    type: Union[Unset, MarketOrderRequestType] = UNSET
    instrument: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, MarketOrderRequestTimeInForce] = UNSET
    price_bound: Union[Unset, str] = UNSET
    position_fill: Union[Unset, MarketOrderRequestPositionFill] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    take_profit_on_fill: Union[Unset, "TakeProfitDetails"] = UNSET
    stop_loss_on_fill: Union[Unset, "StopLossDetails"] = UNSET
    trailing_stop_loss_on_fill: Union[Unset, "TrailingStopLossDetails"] = UNSET
    trade_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        instrument = self.instrument

        units = self.units

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        price_bound = self.price_bound

        position_fill: Union[Unset, str] = UNSET
        if not isinstance(self.position_fill, Unset):
            position_fill = self.position_fill

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        take_profit_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_on_fill, Unset):
            take_profit_on_fill = self.take_profit_on_fill.to_dict()

        stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_on_fill, Unset):
            stop_loss_on_fill = self.stop_loss_on_fill.to_dict()

        trailing_stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = self.trailing_stop_loss_on_fill.to_dict()

        trade_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_client_extensions, Unset):
            trade_client_extensions = self.trade_client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if units is not UNSET:
            field_dict["units"] = units
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if price_bound is not UNSET:
            field_dict["priceBound"] = price_bound
        if position_fill is not UNSET:
            field_dict["positionFill"] = position_fill
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if take_profit_on_fill is not UNSET:
            field_dict["takeProfitOnFill"] = take_profit_on_fill
        if stop_loss_on_fill is not UNSET:
            field_dict["stopLossOnFill"] = stop_loss_on_fill
        if trailing_stop_loss_on_fill is not UNSET:
            field_dict["trailingStopLossOnFill"] = trailing_stop_loss_on_fill
        if trade_client_extensions is not UNSET:
            field_dict["tradeClientExtensions"] = trade_client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trailing_stop_loss_details import TrailingStopLossDetails
        from ..models.take_profit_details import TakeProfitDetails
        from ..models.stop_loss_details import StopLossDetails
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, MarketOrderRequestType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_market_order_request_type(_type)

        instrument = d.pop("instrument", UNSET)

        units = d.pop("units", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, MarketOrderRequestTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_market_order_request_time_in_force(_time_in_force)

        price_bound = d.pop("priceBound", UNSET)

        _position_fill = d.pop("positionFill", UNSET)
        position_fill: Union[Unset, MarketOrderRequestPositionFill]
        if isinstance(_position_fill, Unset):
            position_fill = UNSET
        else:
            position_fill = check_market_order_request_position_fill(_position_fill)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        _take_profit_on_fill = d.pop("takeProfitOnFill", UNSET)
        take_profit_on_fill: Union[Unset, TakeProfitDetails]
        if isinstance(_take_profit_on_fill, Unset):
            take_profit_on_fill = UNSET
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)

        _stop_loss_on_fill = d.pop("stopLossOnFill", UNSET)
        stop_loss_on_fill: Union[Unset, StopLossDetails]
        if isinstance(_stop_loss_on_fill, Unset):
            stop_loss_on_fill = UNSET
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)

        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", UNSET)
        trailing_stop_loss_on_fill: Union[Unset, TrailingStopLossDetails]
        if isinstance(_trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = UNSET
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )

        _trade_client_extensions = d.pop("tradeClientExtensions", UNSET)
        trade_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_trade_client_extensions, Unset):
            trade_client_extensions = UNSET
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )

        market_order_request = cls(
            type=type,
            instrument=instrument,
            units=units,
            time_in_force=time_in_force,
            price_bound=price_bound,
            position_fill=position_fill,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
        )

        market_order_request.additional_properties = d
        return market_order_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

MarketIfTouchedOrderReason = Literal["CLIENT_ORDER", "REPLACEMENT"]

MARKET_IF_TOUCHED_ORDER_REASON_VALUES: Set[MarketIfTouchedOrderReason] = {
    "CLIENT_ORDER",
    "REPLACEMENT",
}


def check_market_if_touched_order_reason(value: str) -> MarketIfTouchedOrderReason:
    if value in MARKET_IF_TOUCHED_ORDER_REASON_VALUES:
        return cast(MarketIfTouchedOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_IF_TOUCHED_ORDER_REASON_VALUES!r}"
    )



from typing import Literal, Set, cast

GuaranteedStopLossOrderMode = Literal["ALLOWED", "DISABLED", "REQUIRED"]

GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES: Set[GuaranteedStopLossOrderMode] = {
    "ALLOWED",
    "DISABLED",
    "REQUIRED",
}


def check_guaranteed_stop_loss_order_mode(value: str) -> GuaranteedStopLossOrderMode:
    if value in GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES:
        return cast(GuaranteedStopLossOrderMode, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES!r}"
    )



from typing import Literal, Set, cast

AccountSummaryGuaranteedStopLossOrderMode = Literal["ALLOWED", "DISABLED", "REQUIRED"]

ACCOUNT_SUMMARY_GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES: Set[
    AccountSummaryGuaranteedStopLossOrderMode
] = {
    "ALLOWED",
    "DISABLED",
    "REQUIRED",
}


def check_account_summary_guaranteed_stop_loss_order_mode(
    value: str,
) -> AccountSummaryGuaranteedStopLossOrderMode:
    if value in ACCOUNT_SUMMARY_GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES:
        return cast(AccountSummaryGuaranteedStopLossOrderMode, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ACCOUNT_SUMMARY_GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES!r}"
    )



from typing import Literal, Set, cast

StopLossOrderRejectTransactionTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]

STOP_LOSS_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES: Set[
    StopLossOrderRejectTransactionTimeInForce
] = {
    "FOK",
    "GFD",
    "GTC",
    "GTD",
    "IOC",
}


def check_stop_loss_order_reject_transaction_time_in_force(
    value: str,
) -> StopLossOrderRejectTransactionTimeInForce:
    if value in STOP_LOSS_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES:
        return cast(StopLossOrderRejectTransactionTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_LOSS_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.delayed_trade_closure_transaction_reason import (
    check_delayed_trade_closure_transaction_reason,
)
from ..models.delayed_trade_closure_transaction_reason import (
    DelayedTradeClosureTransactionReason,
)
from ..models.delayed_trade_closure_transaction_type import (
    check_delayed_trade_closure_transaction_type,
)
from ..models.delayed_trade_closure_transaction_type import (
    DelayedTradeClosureTransactionType,
)
from typing import Union


T = TypeVar("T", bound="DelayedTradeClosureTransaction")


@_attrs_define
class DelayedTradeClosureTransaction:
    """A DelayedTradeClosure Transaction is created administratively to indicate open trades that should have been closed
    but weren't because the open trades' instruments were untradeable at the time. Open trades listed in this
    transaction will be closed once their respective instruments become tradeable.

        Attributes:
            id (Union[Unset, str]): The Transaction's Identifier.
            time (Union[Unset, str]): The date/time when the Transaction was created.
            user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
            account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
            batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
                batch are applied to the Account simultaneously.
            request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
            type (Union[Unset, DelayedTradeClosureTransactionType]): The Type of the Transaction. Always set to
                "DELAYED_TRADE_CLOSURE" for an DelayedTradeClosureTransaction.
            reason (Union[Unset, DelayedTradeClosureTransactionReason]): The reason for the delayed trade closure
            trade_i_ds (Union[Unset, str]): List of Trade ID's identifying the open trades that will be closed when their
                respective instruments become tradeable
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, DelayedTradeClosureTransactionType] = UNSET
    reason: Union[Unset, DelayedTradeClosureTransactionReason] = UNSET
    trade_i_ds: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        trade_i_ds = self.trade_i_ds

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if reason is not UNSET:
            field_dict["reason"] = reason
        if trade_i_ds is not UNSET:
            field_dict["tradeIDs"] = trade_i_ds

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, DelayedTradeClosureTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_delayed_trade_closure_transaction_type(_type)

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, DelayedTradeClosureTransactionReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_delayed_trade_closure_transaction_reason(_reason)

        trade_i_ds = d.pop("tradeIDs", UNSET)

        delayed_trade_closure_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            reason=reason,
            trade_i_ds=trade_i_ds,
        )

        delayed_trade_closure_transaction.additional_properties = d
        return delayed_trade_closure_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

MarketIfTouchedOrderRequestPositionFill = Literal[
    "DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"
]

MARKET_IF_TOUCHED_ORDER_REQUEST_POSITION_FILL_VALUES: Set[
    MarketIfTouchedOrderRequestPositionFill
] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_market_if_touched_order_request_position_fill(
    value: str,
) -> MarketIfTouchedOrderRequestPositionFill:
    if value in MARKET_IF_TOUCHED_ORDER_REQUEST_POSITION_FILL_VALUES:
        return cast(MarketIfTouchedOrderRequestPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_IF_TOUCHED_ORDER_REQUEST_POSITION_FILL_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="OrderIdentifier")


@_attrs_define
class OrderIdentifier:
    """An OrderIdentifier is used to refer to an Order, and contains both the OrderID and the ClientOrderID.

    Attributes:
        order_id (Union[Unset, str]): The OANDA-assigned Order ID
        client_order_id (Union[Unset, str]): The client-provided client Order ID
    """

    order_id: Union[Unset, str] = UNSET
    client_order_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        order_id = self.order_id

        client_order_id = self.client_order_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if order_id is not UNSET:
            field_dict["orderID"] = order_id
        if client_order_id is not UNSET:
            field_dict["clientOrderID"] = client_order_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        order_id = d.pop("orderID", UNSET)

        client_order_id = d.pop("clientOrderID", UNSET)

        order_identifier = cls(
            order_id=order_id,
            client_order_id=client_order_id,
        )

        order_identifier.additional_properties = d
        return order_identifier

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="CandlestickData")


@_attrs_define
class CandlestickData:
    """The price data (open, high, low, close) for the Candlestick representation.

    Attributes:
        o (Union[Unset, str]): The first (open) price in the time-range represented by the candlestick.
        h (Union[Unset, str]): The highest price in the time-range represented by the candlestick.
        l (Union[Unset, str]): The lowest price in the time-range represented by the candlestick.
        c (Union[Unset, str]): The last (closing) price in the time-range represented by the candlestick.
    """

    o: Union[Unset, str] = UNSET
    h: Union[Unset, str] = UNSET
    l: Union[Unset, str] = UNSET
    c: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        o = self.o

        h = self.h

        l = self.l

        c = self.c

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if o is not UNSET:
            field_dict["o"] = o
        if h is not UNSET:
            field_dict["h"] = h
        if l is not UNSET:
            field_dict["l"] = l
        if c is not UNSET:
            field_dict["c"] = c

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        o = d.pop("o", UNSET)

        h = d.pop("h", UNSET)

        l = d.pop("l", UNSET)

        c = d.pop("c", UNSET)

        candlestick_data = cls(
            o=o,
            h=h,
            l=l,
            c=c,
        )

        candlestick_data.additional_properties = d
        return candlestick_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="OrderBookBucket")


@_attrs_define
class OrderBookBucket:
    """The order book data for a partition of the instrument's prices.

    Attributes:
        price (Union[Unset, str]): The lowest price (inclusive) covered by the bucket. The bucket covers the price range
            from the price to price + the order book's bucketWidth.
        long_count_percent (Union[Unset, str]): The percentage of the total number of orders represented by the long
            orders found in this bucket.
        short_count_percent (Union[Unset, str]): The percentage of the total number of orders represented by the short
            orders found in this bucket.
    """

    price: Union[Unset, str] = UNSET
    long_count_percent: Union[Unset, str] = UNSET
    short_count_percent: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        price = self.price

        long_count_percent = self.long_count_percent

        short_count_percent = self.short_count_percent

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if price is not UNSET:
            field_dict["price"] = price
        if long_count_percent is not UNSET:
            field_dict["longCountPercent"] = long_count_percent
        if short_count_percent is not UNSET:
            field_dict["shortCountPercent"] = short_count_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        price = d.pop("price", UNSET)

        long_count_percent = d.pop("longCountPercent", UNSET)

        short_count_percent = d.pop("shortCountPercent", UNSET)

        order_book_bucket = cls(
            price=price,
            long_count_percent=long_count_percent,
            short_count_percent=short_count_percent,
        )

        order_book_bucket.additional_properties = d
        return order_book_bucket

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

MarketOrderReason = Literal[
    "CLIENT_ORDER",
    "DELAYED_TRADE_CLOSE",
    "MARGIN_CLOSEOUT",
    "POSITION_CLOSEOUT",
    "TRADE_CLOSE",
]

MARKET_ORDER_REASON_VALUES: Set[MarketOrderReason] = {
    "CLIENT_ORDER",
    "DELAYED_TRADE_CLOSE",
    "MARGIN_CLOSEOUT",
    "POSITION_CLOSEOUT",
    "TRADE_CLOSE",
}


def check_market_order_reason(value: str) -> MarketOrderReason:
    if value in MARKET_ORDER_REASON_VALUES:
        return cast(MarketOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_ORDER_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.order_state import check_order_state
from ..models.order_state import OrderState
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="Order")


@_attrs_define
class Order:
    """The base Order definition specifies the properties that are common to all Orders.

    Attributes:
        id (Union[Unset, str]): The Order's identifier, unique within the Order's Account.
        create_time (Union[Unset, str]): The time when the Order was created.
        state (Union[Unset, OrderState]): The current state of the Order.
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
    """

    id: Union[Unset, str] = UNSET
    create_time: Union[Unset, str] = UNSET
    state: Union[Unset, OrderState] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        create_time = self.create_time

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if create_time is not UNSET:
            field_dict["createTime"] = create_time
        if state is not UNSET:
            field_dict["state"] = state
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        create_time = d.pop("createTime", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, OrderState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_order_state(_state)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
        )

        order.additional_properties = d
        return order

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

WeeklyAlignment = Literal[
    "Friday", "Monday", "Saturday", "Sunday", "Thursday", "Tuesday", "Wednesday"
]

WEEKLY_ALIGNMENT_VALUES: Set[WeeklyAlignment] = {
    "Friday",
    "Monday",
    "Saturday",
    "Sunday",
    "Thursday",
    "Tuesday",
    "Wednesday",
}


def check_weekly_alignment(value: str) -> WeeklyAlignment:
    if value in WEEKLY_ALIGNMENT_VALUES:
        return cast(WeeklyAlignment, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {WEEKLY_ALIGNMENT_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.order_cancel_transaction import OrderCancelTransaction
    from ..models.order_fill_transaction import OrderFillTransaction
    from ..models.transaction import Transaction


T = TypeVar("T", bound="CreateOrderResponse201")


@_attrs_define
class CreateOrderResponse201:
    """
    Attributes:
        order_create_transaction (Union[Unset, Transaction]): The base Transaction specification. Specifies properties
            that are common between all Transaction.
        order_fill_transaction (Union[Unset, OrderFillTransaction]): An OrderFillTransaction represents the filling of
            an Order in the client's Account.
        order_cancel_transaction (Union[Unset, OrderCancelTransaction]): An OrderCancelTransaction represents the
            cancellation of an Order in the client's Account.
        order_reissue_transaction (Union[Unset, Transaction]): The base Transaction specification. Specifies properties
            that are common between all Transaction.
        order_reissue_reject_transaction (Union[Unset, Transaction]): The base Transaction specification. Specifies
            properties that are common between all Transaction.
        related_transaction_i_ds (Union[Unset, List[str]]): The IDs of all Transactions that were created while
            satisfying the request.
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account
    """

    order_create_transaction: Union[Unset, "Transaction"] = UNSET
    order_fill_transaction: Union[Unset, "OrderFillTransaction"] = UNSET
    order_cancel_transaction: Union[Unset, "OrderCancelTransaction"] = UNSET
    order_reissue_transaction: Union[Unset, "Transaction"] = UNSET
    order_reissue_reject_transaction: Union[Unset, "Transaction"] = UNSET
    related_transaction_i_ds: Union[Unset, List[str]] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        order_create_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order_create_transaction, Unset):
            order_create_transaction = self.order_create_transaction.to_dict()

        order_fill_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order_fill_transaction, Unset):
            order_fill_transaction = self.order_fill_transaction.to_dict()

        order_cancel_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order_cancel_transaction, Unset):
            order_cancel_transaction = self.order_cancel_transaction.to_dict()

        order_reissue_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order_reissue_transaction, Unset):
            order_reissue_transaction = self.order_reissue_transaction.to_dict()

        order_reissue_reject_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order_reissue_reject_transaction, Unset):
            order_reissue_reject_transaction = (
                self.order_reissue_reject_transaction.to_dict()
            )

        related_transaction_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.related_transaction_i_ds, Unset):
            related_transaction_i_ds = self.related_transaction_i_ds

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if order_create_transaction is not UNSET:
            field_dict["orderCreateTransaction"] = order_create_transaction
        if order_fill_transaction is not UNSET:
            field_dict["orderFillTransaction"] = order_fill_transaction
        if order_cancel_transaction is not UNSET:
            field_dict["orderCancelTransaction"] = order_cancel_transaction
        if order_reissue_transaction is not UNSET:
            field_dict["orderReissueTransaction"] = order_reissue_transaction
        if order_reissue_reject_transaction is not UNSET:
            field_dict["orderReissueRejectTransaction"] = (
                order_reissue_reject_transaction
            )
        if related_transaction_i_ds is not UNSET:
            field_dict["relatedTransactionIDs"] = related_transaction_i_ds
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.order_cancel_transaction import OrderCancelTransaction
        from ..models.order_fill_transaction import OrderFillTransaction
        from ..models.transaction import Transaction

        d = src_dict.copy()
        _order_create_transaction = d.pop("orderCreateTransaction", UNSET)
        order_create_transaction: Union[Unset, Transaction]
        if isinstance(_order_create_transaction, Unset):
            order_create_transaction = UNSET
        else:
            order_create_transaction = Transaction.from_dict(_order_create_transaction)

        _order_fill_transaction = d.pop("orderFillTransaction", UNSET)
        order_fill_transaction: Union[Unset, OrderFillTransaction]
        if isinstance(_order_fill_transaction, Unset):
            order_fill_transaction = UNSET
        else:
            order_fill_transaction = OrderFillTransaction.from_dict(
                _order_fill_transaction
            )

        _order_cancel_transaction = d.pop("orderCancelTransaction", UNSET)
        order_cancel_transaction: Union[Unset, OrderCancelTransaction]
        if isinstance(_order_cancel_transaction, Unset):
            order_cancel_transaction = UNSET
        else:
            order_cancel_transaction = OrderCancelTransaction.from_dict(
                _order_cancel_transaction
            )

        _order_reissue_transaction = d.pop("orderReissueTransaction", UNSET)
        order_reissue_transaction: Union[Unset, Transaction]
        if isinstance(_order_reissue_transaction, Unset):
            order_reissue_transaction = UNSET
        else:
            order_reissue_transaction = Transaction.from_dict(
                _order_reissue_transaction
            )

        _order_reissue_reject_transaction = d.pop(
            "orderReissueRejectTransaction", UNSET
        )
        order_reissue_reject_transaction: Union[Unset, Transaction]
        if isinstance(_order_reissue_reject_transaction, Unset):
            order_reissue_reject_transaction = UNSET
        else:
            order_reissue_reject_transaction = Transaction.from_dict(
                _order_reissue_reject_transaction
            )

        related_transaction_i_ds = cast(
            List[str], d.pop("relatedTransactionIDs", UNSET)
        )

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        create_order_response_201 = cls(
            order_create_transaction=order_create_transaction,
            order_fill_transaction=order_fill_transaction,
            order_cancel_transaction=order_cancel_transaction,
            order_reissue_transaction=order_reissue_transaction,
            order_reissue_reject_transaction=order_reissue_reject_transaction,
            related_transaction_i_ds=related_transaction_i_ds,
            last_transaction_id=last_transaction_id,
        )

        create_order_response_201.additional_properties = d
        return create_order_response_201

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.get_prices_response_200 import GetPricesResponse200
from ...types import Unset


def _get_kwargs(
    account_id: str,
    *,
    instruments: List[str],
    since: Union[Unset, str] = UNSET,
    include_units_available: Union[Unset, bool] = UNSET,
    include_home_conversions: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["Authorization"] = authorization

    if not isinstance(accept_datetime_format, Unset):
        headers["Accept-Datetime-Format"] = accept_datetime_format

    params: Dict[str, Any] = {}

    json_instruments = instruments

    params["instruments"] = json_instruments

    params["since"] = since

    params["includeUnitsAvailable"] = include_units_available

    params["includeHomeConversions"] = include_home_conversions

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/accounts/{account_id}/pricing".format(
            account_id=account_id,
        ),
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, GetPricesResponse200]]:
    if response.status_code == 200:
        response_200 = GetPricesResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 405:
        response_405 = cast(Any, None)
        return response_405
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, GetPricesResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    since: Union[Unset, str] = UNSET,
    include_units_available: Union[Unset, bool] = UNSET,
    include_home_conversions: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, GetPricesResponse200]]:
    """Current Account Prices

     Get pricing information for a specified list of Instruments within an Account.

    Args:
        account_id (str):
        instruments (List[str]):
        since (Union[Unset, str]):
        include_units_available (Union[Unset, bool]):
        include_home_conversions (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, GetPricesResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        instruments=instruments,
        since=since,
        include_units_available=include_units_available,
        include_home_conversions=include_home_conversions,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    since: Union[Unset, str] = UNSET,
    include_units_available: Union[Unset, bool] = UNSET,
    include_home_conversions: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, GetPricesResponse200]]:
    """Current Account Prices

     Get pricing information for a specified list of Instruments within an Account.

    Args:
        account_id (str):
        instruments (List[str]):
        since (Union[Unset, str]):
        include_units_available (Union[Unset, bool]):
        include_home_conversions (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, GetPricesResponse200]
    """

    return sync_detailed(
        account_id=account_id,
        client=client,
        instruments=instruments,
        since=since,
        include_units_available=include_units_available,
        include_home_conversions=include_home_conversions,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    since: Union[Unset, str] = UNSET,
    include_units_available: Union[Unset, bool] = UNSET,
    include_home_conversions: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, GetPricesResponse200]]:
    """Current Account Prices

     Get pricing information for a specified list of Instruments within an Account.

    Args:
        account_id (str):
        instruments (List[str]):
        since (Union[Unset, str]):
        include_units_available (Union[Unset, bool]):
        include_home_conversions (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, GetPricesResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        instruments=instruments,
        since=since,
        include_units_available=include_units_available,
        include_home_conversions=include_home_conversions,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    since: Union[Unset, str] = UNSET,
    include_units_available: Union[Unset, bool] = UNSET,
    include_home_conversions: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, GetPricesResponse200]]:
    """Current Account Prices

     Get pricing information for a specified list of Instruments within an Account.

    Args:
        account_id (str):
        instruments (List[str]):
        since (Union[Unset, str]):
        include_units_available (Union[Unset, bool]):
        include_home_conversions (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, GetPricesResponse200]
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            client=client,
            instruments=instruments,
            since=since,
            include_units_available=include_units_available,
            include_home_conversions=include_home_conversions,
            authorization=authorization,
            accept_datetime_format=accept_datetime_format,
        )
    ).parsed



from typing import Literal, Set, cast

LimitOrderReason = Literal["CLIENT_ORDER", "REPLACEMENT"]

LIMIT_ORDER_REASON_VALUES: Set[LimitOrderReason] = {
    "CLIENT_ORDER",
    "REPLACEMENT",
}


def check_limit_order_reason(value: str) -> LimitOrderReason:
    if value in LIMIT_ORDER_REASON_VALUES:
        return cast(LimitOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="UserInfoExternal")


@_attrs_define
class UserInfoExternal:
    """A representation of user information, as available to external (3rd party) clients.

    Attributes:
        user_id (Union[Unset, int]): The user's OANDA-assigned user ID.
        country (Union[Unset, str]): The country that the user is based in.
        fifo (Union[Unset, bool]): Flag indicating if the the user's Accounts adhere to FIFO execution rules.
    """

    user_id: Union[Unset, int] = UNSET
    country: Union[Unset, str] = UNSET
    fifo: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id

        country = self.country

        fifo = self.fifo

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if country is not UNSET:
            field_dict["country"] = country
        if fifo is not UNSET:
            field_dict["FIFO"] = fifo

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("userID", UNSET)

        country = d.pop("country", UNSET)

        fifo = d.pop("FIFO", UNSET)

        user_info_external = cls(
            user_id=user_id,
            country=country,
            fifo=fifo,
        )

        user_info_external.additional_properties = d
        return user_info_external

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

TradeSummaryState = Literal["CLOSE_WHEN_TRADEABLE", "CLOSED", "OPEN"]

TRADE_SUMMARY_STATE_VALUES: Set[TradeSummaryState] = {
    "CLOSE_WHEN_TRADEABLE",
    "CLOSED",
    "OPEN",
}


def check_trade_summary_state(value: str) -> TradeSummaryState:
    if value in TRADE_SUMMARY_STATE_VALUES:
        return cast(TradeSummaryState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRADE_SUMMARY_STATE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="ConfigureAccountBody")


@_attrs_define
class ConfigureAccountBody:
    """
    Attributes:
        alias (Union[Unset, str]): Client-defined alias (name) for the Account
        margin_rate (Union[Unset, str]): The string representation of a decimal number.
    """

    alias: Union[Unset, str] = UNSET
    margin_rate: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        alias = self.alias

        margin_rate = self.margin_rate

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if alias is not UNSET:
            field_dict["alias"] = alias
        if margin_rate is not UNSET:
            field_dict["marginRate"] = margin_rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        alias = d.pop("alias", UNSET)

        margin_rate = d.pop("marginRate", UNSET)

        configure_account_body = cls(
            alias=alias,
            margin_rate=margin_rate,
        )

        configure_account_body.additional_properties = d
        return configure_account_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="TradeOpen")


@_attrs_define
class TradeOpen:
    """A TradeOpen object represents a Trade for an instrument that was opened in an Account. It is found embedded in
    Transactions that affect the position of an instrument in the Account, specifically the OrderFill Transaction.

        Attributes:
            trade_id (Union[Unset, str]): The ID of the Trade that was opened
            units (Union[Unset, str]): The number of units opened by the Trade
            price (Union[Unset, str]): The average price that the units were opened at.
            guaranteed_execution_fee (Union[Unset, str]): This is the fee charged for opening the trade if it has a
                guaranteed Stop Loss Order attached to it.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            half_spread_cost (Union[Unset, str]): The half spread cost for the trade open. This can be a positive or
                negative value and is represented in the home currency of the Account.
            initial_margin_required (Union[Unset, str]): The margin required at the time the Trade was created. Note, this
                is the 'pure' margin required, it is not the 'effective' margin used that factors in the trade risk if a GSLO is
                attached to the trade.
    """

    trade_id: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    guaranteed_execution_fee: Union[Unset, str] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    half_spread_cost: Union[Unset, str] = UNSET
    initial_margin_required: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        trade_id = self.trade_id

        units = self.units

        price = self.price

        guaranteed_execution_fee = self.guaranteed_execution_fee

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        half_spread_cost = self.half_spread_cost

        initial_margin_required = self.initial_margin_required

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if units is not UNSET:
            field_dict["units"] = units
        if price is not UNSET:
            field_dict["price"] = price
        if guaranteed_execution_fee is not UNSET:
            field_dict["guaranteedExecutionFee"] = guaranteed_execution_fee
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if half_spread_cost is not UNSET:
            field_dict["halfSpreadCost"] = half_spread_cost
        if initial_margin_required is not UNSET:
            field_dict["initialMarginRequired"] = initial_margin_required

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        trade_id = d.pop("tradeID", UNSET)

        units = d.pop("units", UNSET)

        price = d.pop("price", UNSET)

        guaranteed_execution_fee = d.pop("guaranteedExecutionFee", UNSET)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        half_spread_cost = d.pop("halfSpreadCost", UNSET)

        initial_margin_required = d.pop("initialMarginRequired", UNSET)

        trade_open = cls(
            trade_id=trade_id,
            units=units,
            price=price,
            guaranteed_execution_fee=guaranteed_execution_fee,
            client_extensions=client_extensions,
            half_spread_cost=half_spread_cost,
            initial_margin_required=initial_margin_required,
        )

        trade_open.additional_properties = d
        return trade_open

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

AcceptDatetimeFormat = Literal["RFC3339", "UNIX"]

ACCEPT_DATETIME_FORMAT_VALUES: Set[AcceptDatetimeFormat] = {
    "RFC3339",
    "UNIX",
}


def check_accept_datetime_format(value: str) -> AcceptDatetimeFormat:
    if value in ACCEPT_DATETIME_FORMAT_VALUES:
        return cast(AcceptDatetimeFormat, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ACCEPT_DATETIME_FORMAT_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union


T = TypeVar("T", bound="AccountProperties")


@_attrs_define
class AccountProperties:
    """Properties related to an Account.

    Attributes:
        id (Union[Unset, str]): The Account's identifier
        mt_4_account_id (Union[Unset, int]): The Account's associated MT4 Account ID. This field will not be present if
            the Account is not an MT4 account.
        tags (Union[Unset, List[str]]): The Account's tags
    """

    id: Union[Unset, str] = UNSET
    mt_4_account_id: Union[Unset, int] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        mt_4_account_id = self.mt_4_account_id

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if mt_4_account_id is not UNSET:
            field_dict["mt4AccountID"] = mt_4_account_id
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        mt_4_account_id = d.pop("mt4AccountID", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        account_properties = cls(
            id=id,
            mt_4_account_id=mt_4_account_id,
            tags=tags,
        )

        account_properties.additional_properties = d
        return account_properties

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.market_order_position_fill import check_market_order_position_fill
from ..models.market_order_position_fill import MarketOrderPositionFill
from ..models.market_order_state import check_market_order_state
from ..models.market_order_state import MarketOrderState
from ..models.market_order_time_in_force import check_market_order_time_in_force
from ..models.market_order_time_in_force import MarketOrderTimeInForce
from ..models.market_order_type import check_market_order_type
from ..models.market_order_type import MarketOrderType
from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.trailing_stop_loss_details import TrailingStopLossDetails
    from ..models.market_order_delayed_trade_close import MarketOrderDelayedTradeClose
    from ..models.market_order_trade_close import MarketOrderTradeClose
    from ..models.market_order_position_closeout import MarketOrderPositionCloseout
    from ..models.take_profit_details import TakeProfitDetails
    from ..models.stop_loss_details import StopLossDetails
    from ..models.market_order_margin_closeout import MarketOrderMarginCloseout
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="MarketOrder")


@_attrs_define
class MarketOrder:
    """A MarketOrder is an order that is filled immediately upon creation using the current market price.

    Attributes:
        id (Union[Unset, str]): The Order's identifier, unique within the Order's Account.
        create_time (Union[Unset, str]): The time when the Order was created.
        state (Union[Unset, MarketOrderState]): The current state of the Order.
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        type (Union[Unset, MarketOrderType]): The type of the Order. Always set to "MARKET" for Market Orders.
        instrument (Union[Unset, str]): The Market Order's Instrument.
        units (Union[Unset, str]): The quantity requested to be filled by the Market Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        time_in_force (Union[Unset, MarketOrderTimeInForce]): The time-in-force requested for the Market Order.
            Restricted to FOK or IOC for a MarketOrder.
        price_bound (Union[Unset, str]): The worst price that the client is willing to have the Market Order filled at.
        position_fill (Union[Unset, MarketOrderPositionFill]): Specification of how Positions in the Account are
            modified when the Order is filled.
        trade_close (Union[Unset, MarketOrderTradeClose]): A MarketOrderTradeClose specifies the extensions to a Market
            Order that has been created specifically to close a Trade.
        long_position_closeout (Union[Unset, MarketOrderPositionCloseout]): A MarketOrderPositionCloseout specifies the
            extensions to a Market Order when it has been created to closeout a specific Position.
        short_position_closeout (Union[Unset, MarketOrderPositionCloseout]): A MarketOrderPositionCloseout specifies the
            extensions to a Market Order when it has been created to closeout a specific Position.
        margin_closeout (Union[Unset, MarketOrderMarginCloseout]): Details for the Market Order extensions specific to a
            Market Order placed that is part of a Market Order Margin Closeout in a client's account
        delayed_trade_close (Union[Unset, MarketOrderDelayedTradeClose]): Details for the Market Order extensions
            specific to a Market Order placed with the intent of fully closing a specific open trade that should have
            already been closed but wasn't due to halted market conditions
        take_profit_on_fill (Union[Unset, TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Union[Unset, StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Union[Unset, TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        filling_transaction_id (Union[Unset, str]): ID of the Transaction that filled this Order (only provided when the
            Order's state is FILLED)
        filled_time (Union[Unset, str]): Date/time when the Order was filled (only provided when the Order's state is
            FILLED)
        trade_opened_id (Union[Unset, str]): Trade ID of Trade opened when the Order was filled (only provided when the
            Order's state is FILLED and a Trade was opened as a result of the fill)
        trade_reduced_id (Union[Unset, str]): Trade ID of Trade reduced when the Order was filled (only provided when
            the Order's state is FILLED and a Trade was reduced as a result of the fill)
        trade_closed_i_ds (Union[Unset, List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
            when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
        cancelling_transaction_id (Union[Unset, str]): ID of the Transaction that cancelled the Order (only provided
            when the Order's state is CANCELLED)
        cancelled_time (Union[Unset, str]): Date/time when the Order was cancelled (only provided when the state of the
            Order is CANCELLED)
    """

    id: Union[Unset, str] = UNSET
    create_time: Union[Unset, str] = UNSET
    state: Union[Unset, MarketOrderState] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    type: Union[Unset, MarketOrderType] = UNSET
    instrument: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, MarketOrderTimeInForce] = UNSET
    price_bound: Union[Unset, str] = UNSET
    position_fill: Union[Unset, MarketOrderPositionFill] = UNSET
    trade_close: Union[Unset, "MarketOrderTradeClose"] = UNSET
    long_position_closeout: Union[Unset, "MarketOrderPositionCloseout"] = UNSET
    short_position_closeout: Union[Unset, "MarketOrderPositionCloseout"] = UNSET
    margin_closeout: Union[Unset, "MarketOrderMarginCloseout"] = UNSET
    delayed_trade_close: Union[Unset, "MarketOrderDelayedTradeClose"] = UNSET
    take_profit_on_fill: Union[Unset, "TakeProfitDetails"] = UNSET
    stop_loss_on_fill: Union[Unset, "StopLossDetails"] = UNSET
    trailing_stop_loss_on_fill: Union[Unset, "TrailingStopLossDetails"] = UNSET
    trade_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    filling_transaction_id: Union[Unset, str] = UNSET
    filled_time: Union[Unset, str] = UNSET
    trade_opened_id: Union[Unset, str] = UNSET
    trade_reduced_id: Union[Unset, str] = UNSET
    trade_closed_i_ds: Union[Unset, List[str]] = UNSET
    cancelling_transaction_id: Union[Unset, str] = UNSET
    cancelled_time: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        create_time = self.create_time

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        instrument = self.instrument

        units = self.units

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        price_bound = self.price_bound

        position_fill: Union[Unset, str] = UNSET
        if not isinstance(self.position_fill, Unset):
            position_fill = self.position_fill

        trade_close: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_close, Unset):
            trade_close = self.trade_close.to_dict()

        long_position_closeout: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.long_position_closeout, Unset):
            long_position_closeout = self.long_position_closeout.to_dict()

        short_position_closeout: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.short_position_closeout, Unset):
            short_position_closeout = self.short_position_closeout.to_dict()

        margin_closeout: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.margin_closeout, Unset):
            margin_closeout = self.margin_closeout.to_dict()

        delayed_trade_close: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.delayed_trade_close, Unset):
            delayed_trade_close = self.delayed_trade_close.to_dict()

        take_profit_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_on_fill, Unset):
            take_profit_on_fill = self.take_profit_on_fill.to_dict()

        stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_on_fill, Unset):
            stop_loss_on_fill = self.stop_loss_on_fill.to_dict()

        trailing_stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = self.trailing_stop_loss_on_fill.to_dict()

        trade_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_client_extensions, Unset):
            trade_client_extensions = self.trade_client_extensions.to_dict()

        filling_transaction_id = self.filling_transaction_id

        filled_time = self.filled_time

        trade_opened_id = self.trade_opened_id

        trade_reduced_id = self.trade_reduced_id

        trade_closed_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.trade_closed_i_ds, Unset):
            trade_closed_i_ds = self.trade_closed_i_ds

        cancelling_transaction_id = self.cancelling_transaction_id

        cancelled_time = self.cancelled_time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if create_time is not UNSET:
            field_dict["createTime"] = create_time
        if state is not UNSET:
            field_dict["state"] = state
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if type is not UNSET:
            field_dict["type"] = type
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if units is not UNSET:
            field_dict["units"] = units
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if price_bound is not UNSET:
            field_dict["priceBound"] = price_bound
        if position_fill is not UNSET:
            field_dict["positionFill"] = position_fill
        if trade_close is not UNSET:
            field_dict["tradeClose"] = trade_close
        if long_position_closeout is not UNSET:
            field_dict["longPositionCloseout"] = long_position_closeout
        if short_position_closeout is not UNSET:
            field_dict["shortPositionCloseout"] = short_position_closeout
        if margin_closeout is not UNSET:
            field_dict["marginCloseout"] = margin_closeout
        if delayed_trade_close is not UNSET:
            field_dict["delayedTradeClose"] = delayed_trade_close
        if take_profit_on_fill is not UNSET:
            field_dict["takeProfitOnFill"] = take_profit_on_fill
        if stop_loss_on_fill is not UNSET:
            field_dict["stopLossOnFill"] = stop_loss_on_fill
        if trailing_stop_loss_on_fill is not UNSET:
            field_dict["trailingStopLossOnFill"] = trailing_stop_loss_on_fill
        if trade_client_extensions is not UNSET:
            field_dict["tradeClientExtensions"] = trade_client_extensions
        if filling_transaction_id is not UNSET:
            field_dict["fillingTransactionID"] = filling_transaction_id
        if filled_time is not UNSET:
            field_dict["filledTime"] = filled_time
        if trade_opened_id is not UNSET:
            field_dict["tradeOpenedID"] = trade_opened_id
        if trade_reduced_id is not UNSET:
            field_dict["tradeReducedID"] = trade_reduced_id
        if trade_closed_i_ds is not UNSET:
            field_dict["tradeClosedIDs"] = trade_closed_i_ds
        if cancelling_transaction_id is not UNSET:
            field_dict["cancellingTransactionID"] = cancelling_transaction_id
        if cancelled_time is not UNSET:
            field_dict["cancelledTime"] = cancelled_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trailing_stop_loss_details import TrailingStopLossDetails
        from ..models.market_order_delayed_trade_close import (
            MarketOrderDelayedTradeClose,
        )
        from ..models.market_order_trade_close import MarketOrderTradeClose
        from ..models.market_order_position_closeout import MarketOrderPositionCloseout
        from ..models.take_profit_details import TakeProfitDetails
        from ..models.stop_loss_details import StopLossDetails
        from ..models.market_order_margin_closeout import MarketOrderMarginCloseout
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        create_time = d.pop("createTime", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, MarketOrderState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_market_order_state(_state)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        _type = d.pop("type", UNSET)
        type: Union[Unset, MarketOrderType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_market_order_type(_type)

        instrument = d.pop("instrument", UNSET)

        units = d.pop("units", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, MarketOrderTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_market_order_time_in_force(_time_in_force)

        price_bound = d.pop("priceBound", UNSET)

        _position_fill = d.pop("positionFill", UNSET)
        position_fill: Union[Unset, MarketOrderPositionFill]
        if isinstance(_position_fill, Unset):
            position_fill = UNSET
        else:
            position_fill = check_market_order_position_fill(_position_fill)

        _trade_close = d.pop("tradeClose", UNSET)
        trade_close: Union[Unset, MarketOrderTradeClose]
        if isinstance(_trade_close, Unset):
            trade_close = UNSET
        else:
            trade_close = MarketOrderTradeClose.from_dict(_trade_close)

        _long_position_closeout = d.pop("longPositionCloseout", UNSET)
        long_position_closeout: Union[Unset, MarketOrderPositionCloseout]
        if isinstance(_long_position_closeout, Unset):
            long_position_closeout = UNSET
        else:
            long_position_closeout = MarketOrderPositionCloseout.from_dict(
                _long_position_closeout
            )

        _short_position_closeout = d.pop("shortPositionCloseout", UNSET)
        short_position_closeout: Union[Unset, MarketOrderPositionCloseout]
        if isinstance(_short_position_closeout, Unset):
            short_position_closeout = UNSET
        else:
            short_position_closeout = MarketOrderPositionCloseout.from_dict(
                _short_position_closeout
            )

        _margin_closeout = d.pop("marginCloseout", UNSET)
        margin_closeout: Union[Unset, MarketOrderMarginCloseout]
        if isinstance(_margin_closeout, Unset):
            margin_closeout = UNSET
        else:
            margin_closeout = MarketOrderMarginCloseout.from_dict(_margin_closeout)

        _delayed_trade_close = d.pop("delayedTradeClose", UNSET)
        delayed_trade_close: Union[Unset, MarketOrderDelayedTradeClose]
        if isinstance(_delayed_trade_close, Unset):
            delayed_trade_close = UNSET
        else:
            delayed_trade_close = MarketOrderDelayedTradeClose.from_dict(
                _delayed_trade_close
            )

        _take_profit_on_fill = d.pop("takeProfitOnFill", UNSET)
        take_profit_on_fill: Union[Unset, TakeProfitDetails]
        if isinstance(_take_profit_on_fill, Unset):
            take_profit_on_fill = UNSET
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)

        _stop_loss_on_fill = d.pop("stopLossOnFill", UNSET)
        stop_loss_on_fill: Union[Unset, StopLossDetails]
        if isinstance(_stop_loss_on_fill, Unset):
            stop_loss_on_fill = UNSET
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)

        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", UNSET)
        trailing_stop_loss_on_fill: Union[Unset, TrailingStopLossDetails]
        if isinstance(_trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = UNSET
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )

        _trade_client_extensions = d.pop("tradeClientExtensions", UNSET)
        trade_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_trade_client_extensions, Unset):
            trade_client_extensions = UNSET
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )

        filling_transaction_id = d.pop("fillingTransactionID", UNSET)

        filled_time = d.pop("filledTime", UNSET)

        trade_opened_id = d.pop("tradeOpenedID", UNSET)

        trade_reduced_id = d.pop("tradeReducedID", UNSET)

        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", UNSET))

        cancelling_transaction_id = d.pop("cancellingTransactionID", UNSET)

        cancelled_time = d.pop("cancelledTime", UNSET)

        market_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            instrument=instrument,
            units=units,
            time_in_force=time_in_force,
            price_bound=price_bound,
            position_fill=position_fill,
            trade_close=trade_close,
            long_position_closeout=long_position_closeout,
            short_position_closeout=short_position_closeout,
            margin_closeout=margin_closeout,
            delayed_trade_close=delayed_trade_close,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
        )

        market_order.additional_properties = d
        return market_order

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="QuoteHomeConversionFactors")


@_attrs_define
class QuoteHomeConversionFactors:
    """QuoteHomeConversionFactors represents the factors that can be used used to convert quantities of a Price's
    Instrument's quote currency into the Account's home currency.

        Attributes:
            positive_units (Union[Unset, str]): The factor used to convert a positive amount of the Price's Instrument's
                quote currency into a positive amount of the Account's home currency.  Conversion is performed by multiplying
                the quote units by the conversion factor.
            negative_units (Union[Unset, str]): The factor used to convert a negative amount of the Price's Instrument's
                quote currency into a negative amount of the Account's home currency.  Conversion is performed by multiplying
                the quote units by the conversion factor.
    """

    positive_units: Union[Unset, str] = UNSET
    negative_units: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        positive_units = self.positive_units

        negative_units = self.negative_units

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if positive_units is not UNSET:
            field_dict["positiveUnits"] = positive_units
        if negative_units is not UNSET:
            field_dict["negativeUnits"] = negative_units

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        positive_units = d.pop("positiveUnits", UNSET)

        negative_units = d.pop("negativeUnits", UNSET)

        quote_home_conversion_factors = cls(
            positive_units=positive_units,
            negative_units=negative_units,
        )

        quote_home_conversion_factors.additional_properties = d
        return quote_home_conversion_factors

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="InstrumentCommission")


@_attrs_define
class InstrumentCommission:
    """An InstrumentCommission represents an instrument-specific commission

    Attributes:
        commission (Union[Unset, str]): The commission amount (in the Account's home currency) charged per unitsTraded
            of the instrument
        units_traded (Union[Unset, str]): The number of units traded that the commission amount is based on.
        minimum_commission (Union[Unset, str]): The minimum commission amount (in the Account's home currency) that is
            charged when an Order is filled for this instrument.
    """

    commission: Union[Unset, str] = UNSET
    units_traded: Union[Unset, str] = UNSET
    minimum_commission: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        commission = self.commission

        units_traded = self.units_traded

        minimum_commission = self.minimum_commission

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if commission is not UNSET:
            field_dict["commission"] = commission
        if units_traded is not UNSET:
            field_dict["unitsTraded"] = units_traded
        if minimum_commission is not UNSET:
            field_dict["minimumCommission"] = minimum_commission

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        commission = d.pop("commission", UNSET)

        units_traded = d.pop("unitsTraded", UNSET)

        minimum_commission = d.pop("minimumCommission", UNSET)

        instrument_commission = cls(
            commission=commission,
            units_traded=units_traded,
            minimum_commission=minimum_commission,
        )

        instrument_commission.additional_properties = d
        return instrument_commission

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.client_price_status import check_client_price_status
from ..models.client_price_status import ClientPriceStatus
from typing import Union

if TYPE_CHECKING:
    from ..models.quote_home_conversion_factors import QuoteHomeConversionFactors
    from ..models.units_available import UnitsAvailable
    from ..models.price_bucket import PriceBucket


T = TypeVar("T", bound="ClientPrice")


@_attrs_define
class ClientPrice:
    """The specification of an Account-specific Price.

    Attributes:
        type (Union[Unset, str]): The string "PRICE". Used to identify the a Price object when found in a stream.
        instrument (Union[Unset, str]): The Price's Instrument.
        time (Union[Unset, str]): The date/time when the Price was created
        status (Union[Unset, ClientPriceStatus]): The status of the Price.
        tradeable (Union[Unset, bool]): Flag indicating if the Price is tradeable or not
        bids (Union[Unset, List['PriceBucket']]): The list of prices and liquidity available on the Instrument's bid
            side. It is possible for this list to be empty if there is no bid liquidity currently available for the
            Instrument in the Account.
        asks (Union[Unset, List['PriceBucket']]): The list of prices and liquidity available on the Instrument's ask
            side. It is possible for this list to be empty if there is no ask liquidity currently available for the
            Instrument in the Account.
        closeout_bid (Union[Unset, str]): The closeout bid Price. This Price is used when a bid is required to closeout
            a Position (margin closeout or manual) yet there is no bid liquidity. The closeout bid is never used to open a
            new position.
        closeout_ask (Union[Unset, str]): The closeout ask Price. This Price is used when a ask is required to closeout
            a Position (margin closeout or manual) yet there is no ask liquidity. The closeout ask is never used to open a
            new position.
        quote_home_conversion_factors (Union[Unset, QuoteHomeConversionFactors]): QuoteHomeConversionFactors represents
            the factors that can be used used to convert quantities of a Price's Instrument's quote currency into the
            Account's home currency.
        units_available (Union[Unset, UnitsAvailable]): Representation of how many units of an Instrument are available
            to be traded by an Order depending on its postionFill option.
    """

    type: Union[Unset, str] = UNSET
    instrument: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    status: Union[Unset, ClientPriceStatus] = UNSET
    tradeable: Union[Unset, bool] = UNSET
    bids: Union[Unset, List["PriceBucket"]] = UNSET
    asks: Union[Unset, List["PriceBucket"]] = UNSET
    closeout_bid: Union[Unset, str] = UNSET
    closeout_ask: Union[Unset, str] = UNSET
    quote_home_conversion_factors: Union[Unset, "QuoteHomeConversionFactors"] = UNSET
    units_available: Union[Unset, "UnitsAvailable"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        instrument = self.instrument

        time = self.time

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        tradeable = self.tradeable

        bids: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bids, Unset):
            bids = []
            for bids_item_data in self.bids:
                bids_item = bids_item_data.to_dict()
                bids.append(bids_item)

        asks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.asks, Unset):
            asks = []
            for asks_item_data in self.asks:
                asks_item = asks_item_data.to_dict()
                asks.append(asks_item)

        closeout_bid = self.closeout_bid

        closeout_ask = self.closeout_ask

        quote_home_conversion_factors: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.quote_home_conversion_factors, Unset):
            quote_home_conversion_factors = self.quote_home_conversion_factors.to_dict()

        units_available: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.units_available, Unset):
            units_available = self.units_available.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if time is not UNSET:
            field_dict["time"] = time
        if status is not UNSET:
            field_dict["status"] = status
        if tradeable is not UNSET:
            field_dict["tradeable"] = tradeable
        if bids is not UNSET:
            field_dict["bids"] = bids
        if asks is not UNSET:
            field_dict["asks"] = asks
        if closeout_bid is not UNSET:
            field_dict["closeoutBid"] = closeout_bid
        if closeout_ask is not UNSET:
            field_dict["closeoutAsk"] = closeout_ask
        if quote_home_conversion_factors is not UNSET:
            field_dict["quoteHomeConversionFactors"] = quote_home_conversion_factors
        if units_available is not UNSET:
            field_dict["unitsAvailable"] = units_available

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.quote_home_conversion_factors import QuoteHomeConversionFactors
        from ..models.units_available import UnitsAvailable
        from ..models.price_bucket import PriceBucket

        d = src_dict.copy()
        type = d.pop("type", UNSET)

        instrument = d.pop("instrument", UNSET)

        time = d.pop("time", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ClientPriceStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = check_client_price_status(_status)

        tradeable = d.pop("tradeable", UNSET)

        bids = []
        _bids = d.pop("bids", UNSET)
        for bids_item_data in _bids or []:
            bids_item = PriceBucket.from_dict(bids_item_data)

            bids.append(bids_item)

        asks = []
        _asks = d.pop("asks", UNSET)
        for asks_item_data in _asks or []:
            asks_item = PriceBucket.from_dict(asks_item_data)

            asks.append(asks_item)

        closeout_bid = d.pop("closeoutBid", UNSET)

        closeout_ask = d.pop("closeoutAsk", UNSET)

        _quote_home_conversion_factors = d.pop("quoteHomeConversionFactors", UNSET)
        quote_home_conversion_factors: Union[Unset, QuoteHomeConversionFactors]
        if isinstance(_quote_home_conversion_factors, Unset):
            quote_home_conversion_factors = UNSET
        else:
            quote_home_conversion_factors = QuoteHomeConversionFactors.from_dict(
                _quote_home_conversion_factors
            )

        _units_available = d.pop("unitsAvailable", UNSET)
        units_available: Union[Unset, UnitsAvailable]
        if isinstance(_units_available, Unset):
            units_available = UNSET
        else:
            units_available = UnitsAvailable.from_dict(_units_available)

        client_price = cls(
            type=type,
            instrument=instrument,
            time=time,
            status=status,
            tradeable=tradeable,
            bids=bids,
            asks=asks,
            closeout_bid=closeout_bid,
            closeout_ask=closeout_ask,
            quote_home_conversion_factors=quote_home_conversion_factors,
            units_available=units_available,
        )

        client_price.additional_properties = d
        return client_price

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.list_orders_response_200 import ListOrdersResponse200
from ...types import Unset


def _get_kwargs(
    account_id: str,
    *,
    ids: Union[Unset, List[str]] = UNSET,
    state: Union[Unset, str] = UNSET,
    instrument: Union[Unset, str] = UNSET,
    count: Union[Unset, int] = UNSET,
    before_id: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["Authorization"] = authorization

    if not isinstance(accept_datetime_format, Unset):
        headers["Accept-Datetime-Format"] = accept_datetime_format

    params: Dict[str, Any] = {}

    json_ids: Union[Unset, List[str]] = UNSET
    if not isinstance(ids, Unset):
        json_ids = ids

    params["ids"] = json_ids

    params["state"] = state

    params["instrument"] = instrument

    params["count"] = count

    params["beforeID"] = before_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/accounts/{account_id}/orders".format(
            account_id=account_id,
        ),
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, ListOrdersResponse200]]:
    if response.status_code == 200:
        response_200 = ListOrdersResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 405:
        response_405 = cast(Any, None)
        return response_405
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, ListOrdersResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    ids: Union[Unset, List[str]] = UNSET,
    state: Union[Unset, str] = UNSET,
    instrument: Union[Unset, str] = UNSET,
    count: Union[Unset, int] = UNSET,
    before_id: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, ListOrdersResponse200]]:
    """List Orders

     Get a list of Orders for an Account

    Args:
        account_id (str):
        ids (Union[Unset, List[str]]):
        state (Union[Unset, str]):
        instrument (Union[Unset, str]):
        count (Union[Unset, int]):
        before_id (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ListOrdersResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        ids=ids,
        state=state,
        instrument=instrument,
        count=count,
        before_id=before_id,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    ids: Union[Unset, List[str]] = UNSET,
    state: Union[Unset, str] = UNSET,
    instrument: Union[Unset, str] = UNSET,
    count: Union[Unset, int] = UNSET,
    before_id: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, ListOrdersResponse200]]:
    """List Orders

     Get a list of Orders for an Account

    Args:
        account_id (str):
        ids (Union[Unset, List[str]]):
        state (Union[Unset, str]):
        instrument (Union[Unset, str]):
        count (Union[Unset, int]):
        before_id (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ListOrdersResponse200]
    """

    return sync_detailed(
        account_id=account_id,
        client=client,
        ids=ids,
        state=state,
        instrument=instrument,
        count=count,
        before_id=before_id,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    ids: Union[Unset, List[str]] = UNSET,
    state: Union[Unset, str] = UNSET,
    instrument: Union[Unset, str] = UNSET,
    count: Union[Unset, int] = UNSET,
    before_id: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, ListOrdersResponse200]]:
    """List Orders

     Get a list of Orders for an Account

    Args:
        account_id (str):
        ids (Union[Unset, List[str]]):
        state (Union[Unset, str]):
        instrument (Union[Unset, str]):
        count (Union[Unset, int]):
        before_id (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ListOrdersResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        ids=ids,
        state=state,
        instrument=instrument,
        count=count,
        before_id=before_id,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    ids: Union[Unset, List[str]] = UNSET,
    state: Union[Unset, str] = UNSET,
    instrument: Union[Unset, str] = UNSET,
    count: Union[Unset, int] = UNSET,
    before_id: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, ListOrdersResponse200]]:
    """List Orders

     Get a list of Orders for an Account

    Args:
        account_id (str):
        ids (Union[Unset, List[str]]):
        state (Union[Unset, str]):
        instrument (Union[Unset, str]):
        count (Union[Unset, int]):
        before_id (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ListOrdersResponse200]
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            client=client,
            ids=ids,
            state=state,
            instrument=instrument,
            count=count,
            before_id=before_id,
            authorization=authorization,
            accept_datetime_format=accept_datetime_format,
        )
    ).parsed



from typing import Literal, Set, cast

MarketIfTouchedOrderRequestTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]

MARKET_IF_TOUCHED_ORDER_REQUEST_TIME_IN_FORCE_VALUES: Set[
    MarketIfTouchedOrderRequestTimeInForce
] = {
    "FOK",
    "GFD",
    "GTC",
    "GTD",
    "IOC",
}


def check_market_if_touched_order_request_time_in_force(
    value: str,
) -> MarketIfTouchedOrderRequestTimeInForce:
    if value in MARKET_IF_TOUCHED_ORDER_REQUEST_TIME_IN_FORCE_VALUES:
        return cast(MarketIfTouchedOrderRequestTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_IF_TOUCHED_ORDER_REQUEST_TIME_IN_FORCE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.account_changes import AccountChanges
    from ..models.account_changes_state import AccountChangesState


T = TypeVar("T", bound="GetAccountChangesResponse200")


@_attrs_define
class GetAccountChangesResponse200:
    """
    Attributes:
        changes (Union[Unset, AccountChanges]): An AccountChanges Object is used to represent the changes to an
            Account's Orders, Trades and Positions since a specified Account TransactionID in the past.
        state (Union[Unset, AccountChangesState]): An AccountState Object is used to represent an Account's current
            price-dependent state. Price-dependent Account state is dependent on OANDA's current Prices, and includes things
            like unrealized PL, NAV and Trailing Stop Loss Order state.
        last_transaction_id (Union[Unset, str]): The ID of the last Transaction created for the Account.  This
            Transaction ID should be used for future poll requests, as the client has already observed all changes up to and
            including it.
    """

    changes: Union[Unset, "AccountChanges"] = UNSET
    state: Union[Unset, "AccountChangesState"] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        changes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.changes, Unset):
            changes = self.changes.to_dict()

        state: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.to_dict()

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if changes is not UNSET:
            field_dict["changes"] = changes
        if state is not UNSET:
            field_dict["state"] = state
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.account_changes import AccountChanges
        from ..models.account_changes_state import AccountChangesState

        d = src_dict.copy()
        _changes = d.pop("changes", UNSET)
        changes: Union[Unset, AccountChanges]
        if isinstance(_changes, Unset):
            changes = UNSET
        else:
            changes = AccountChanges.from_dict(_changes)

        _state = d.pop("state", UNSET)
        state: Union[Unset, AccountChangesState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = AccountChangesState.from_dict(_state)

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        get_account_changes_response_200 = cls(
            changes=changes,
            state=state,
            last_transaction_id=last_transaction_id,
        )

        get_account_changes_response_200.additional_properties = d
        return get_account_changes_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

StopLossDetailsTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]

STOP_LOSS_DETAILS_TIME_IN_FORCE_VALUES: Set[StopLossDetailsTimeInForce] = {
    "FOK",
    "GFD",
    "GTC",
    "GTD",
    "IOC",
}


def check_stop_loss_details_time_in_force(value: str) -> StopLossDetailsTimeInForce:
    if value in STOP_LOSS_DETAILS_TIME_IN_FORCE_VALUES:
        return cast(StopLossDetailsTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_LOSS_DETAILS_TIME_IN_FORCE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="HomeConversions")


@_attrs_define
class HomeConversions:
    """HomeConversions represents the factors to use to convert quantities of a given currency into the Account's home
    currency. The conversion factor depends on the scenario the conversion is required for.

        Attributes:
            currency (Union[Unset, str]): The currency to be converted into the home currency.
            account_gain (Union[Unset, str]): The factor used to convert any gains for an Account in the specified currency
                into the Account's home currency. This would include positive realized P/L and positive financing amounts.
                Conversion is performed by multiplying the positive P/L by the conversion factor.
            account_loss (Union[Unset, str]): The string representation of a decimal number.
            position_value (Union[Unset, str]): The factor used to convert a Position or Trade Value in the specified
                currency into the Account's home currency. Conversion is performed by multiplying the Position or Trade Value by
                the conversion factor.
    """

    currency: Union[Unset, str] = UNSET
    account_gain: Union[Unset, str] = UNSET
    account_loss: Union[Unset, str] = UNSET
    position_value: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        currency = self.currency

        account_gain = self.account_gain

        account_loss = self.account_loss

        position_value = self.position_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if currency is not UNSET:
            field_dict["currency"] = currency
        if account_gain is not UNSET:
            field_dict["accountGain"] = account_gain
        if account_loss is not UNSET:
            field_dict["accountLoss"] = account_loss
        if position_value is not UNSET:
            field_dict["positionValue"] = position_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        currency = d.pop("currency", UNSET)

        account_gain = d.pop("accountGain", UNSET)

        account_loss = d.pop("accountLoss", UNSET)

        position_value = d.pop("positionValue", UNSET)

        home_conversions = cls(
            currency=currency,
            account_gain=account_gain,
            account_loss=account_loss,
            position_value=position_value,
        )

        home_conversions.additional_properties = d
        return home_conversions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.liquidity_regeneration_schedule_step import (
        LiquidityRegenerationScheduleStep,
    )


T = TypeVar("T", bound="LiquidityRegenerationSchedule")


@_attrs_define
class LiquidityRegenerationSchedule:
    """A LiquidityRegenerationSchedule indicates how liquidity that is used when filling an Order for an instrument is
    regenerated following the fill.  A liquidity regeneration schedule will be in effect until the timestamp of its
    final step, but may be replaced by a schedule created for an Order of the same instrument that is filled while it is
    still in effect.

        Attributes:
            steps (Union[Unset, List['LiquidityRegenerationScheduleStep']]): The steps in the Liquidity Regeneration
                Schedule
    """

    steps: Union[Unset, List["LiquidityRegenerationScheduleStep"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        steps: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.steps, Unset):
            steps = []
            for steps_item_data in self.steps:
                steps_item = steps_item_data.to_dict()
                steps.append(steps_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if steps is not UNSET:
            field_dict["steps"] = steps

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.liquidity_regeneration_schedule_step import (
            LiquidityRegenerationScheduleStep,
        )

        d = src_dict.copy()
        steps = []
        _steps = d.pop("steps", UNSET)
        for steps_item_data in _steps or []:
            steps_item = LiquidityRegenerationScheduleStep.from_dict(steps_item_data)

            steps.append(steps_item)

        liquidity_regeneration_schedule = cls(
            steps=steps,
        )

        liquidity_regeneration_schedule.additional_properties = d
        return liquidity_regeneration_schedule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

TradeStateFilter = Literal["ALL", "CLOSE_WHEN_TRADEABLE", "CLOSED", "OPEN"]

TRADE_STATE_FILTER_VALUES: Set[TradeStateFilter] = {
    "ALL",
    "CLOSE_WHEN_TRADEABLE",
    "CLOSED",
    "OPEN",
}


def check_trade_state_filter(value: str) -> TradeStateFilter:
    if value in TRADE_STATE_FILTER_VALUES:
        return cast(TradeStateFilter, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRADE_STATE_FILTER_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="ClosePositionBody")


@_attrs_define
class ClosePositionBody:
    """
    Attributes:
        long_units (Union[Unset, str]): Indication of how much of the long Position to closeout. Either the string
            "ALL", the string "NONE", or a DecimalNumber representing how many units of the long position to close using a
            PositionCloseout MarketOrder. The units specified must always be positive.
        long_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        short_units (Union[Unset, str]): Indication of how much of the short Position to closeout. Either the string
            "ALL", the string "NONE", or a DecimalNumber representing how many units of the short position to close using a
            PositionCloseout MarketOrder. The units specified must always be positive.
        short_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
    """

    long_units: Union[Unset, str] = UNSET
    long_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    short_units: Union[Unset, str] = UNSET
    short_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        long_units = self.long_units

        long_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.long_client_extensions, Unset):
            long_client_extensions = self.long_client_extensions.to_dict()

        short_units = self.short_units

        short_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.short_client_extensions, Unset):
            short_client_extensions = self.short_client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if long_units is not UNSET:
            field_dict["longUnits"] = long_units
        if long_client_extensions is not UNSET:
            field_dict["longClientExtensions"] = long_client_extensions
        if short_units is not UNSET:
            field_dict["shortUnits"] = short_units
        if short_client_extensions is not UNSET:
            field_dict["shortClientExtensions"] = short_client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        long_units = d.pop("longUnits", UNSET)

        _long_client_extensions = d.pop("longClientExtensions", UNSET)
        long_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_long_client_extensions, Unset):
            long_client_extensions = UNSET
        else:
            long_client_extensions = ClientExtensions.from_dict(_long_client_extensions)

        short_units = d.pop("shortUnits", UNSET)

        _short_client_extensions = d.pop("shortClientExtensions", UNSET)
        short_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_short_client_extensions, Unset):
            short_client_extensions = UNSET
        else:
            short_client_extensions = ClientExtensions.from_dict(
                _short_client_extensions
            )

        close_position_body = cls(
            long_units=long_units,
            long_client_extensions=long_client_extensions,
            short_units=short_units,
            short_client_extensions=short_client_extensions,
        )

        close_position_body.additional_properties = d
        return close_position_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.account_summary import AccountSummary


T = TypeVar("T", bound="GetAccountSummaryResponse200")


@_attrs_define
class GetAccountSummaryResponse200:
    """
    Attributes:
        account (Union[Unset, AccountSummary]): A summary representation of a client's Account. The AccountSummary does
            not provide to full specification of pending Orders, open Trades and Positions.
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account.
    """

    account: Union[Unset, "AccountSummary"] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        account: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.account, Unset):
            account = self.account.to_dict()

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if account is not UNSET:
            field_dict["account"] = account
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.account_summary import AccountSummary

        d = src_dict.copy()
        _account = d.pop("account", UNSET)
        account: Union[Unset, AccountSummary]
        if isinstance(_account, Unset):
            account = UNSET
        else:
            account = AccountSummary.from_dict(_account)

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        get_account_summary_response_200 = cls(
            account=account,
            last_transaction_id=last_transaction_id,
        )

        get_account_summary_response_200.additional_properties = d
        return get_account_summary_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.stop_loss_details_time_in_force import (
    check_stop_loss_details_time_in_force,
)
from ..models.stop_loss_details_time_in_force import StopLossDetailsTimeInForce
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="StopLossDetails")


@_attrs_define
class StopLossDetails:
    """StopLossDetails specifies the details of a Stop Loss Order to be created on behalf of a client. This may happen when
    an Order is filled that opens a Trade requiring a Stop Loss, or when a Trade's dependent Stop Loss Order is modified
    directly through the Trade.

        Attributes:
            price (Union[Unset, str]): The price that the Stop Loss Order will be triggered at. Only one of the price and
                distance fields may be specified.
            distance (Union[Unset, str]): Specifies the distance (in price units) from the Trade's open price to use as the
                Stop Loss Order price. Only one of the distance and price fields may be specified.
            time_in_force (Union[Unset, StopLossDetailsTimeInForce]): The time in force for the created Stop Loss Order.
                This may only be GTC, GTD or GFD.
            gtd_time (Union[Unset, str]): The date when the Stop Loss Order will be cancelled on if timeInForce is GTD.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            guaranteed (Union[Unset, bool]): Flag indicating that the price for the Stop Loss Order is guaranteed. The
                default value depends on the GuaranteedStopLossOrderMode of the account, if it is REQUIRED, the default will be
                true, for DISABLED or ENABLED the default is false.
    """

    price: Union[Unset, str] = UNSET
    distance: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, StopLossDetailsTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    guaranteed: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        price = self.price

        distance = self.distance

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        guaranteed = self.guaranteed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if price is not UNSET:
            field_dict["price"] = price
        if distance is not UNSET:
            field_dict["distance"] = distance
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if guaranteed is not UNSET:
            field_dict["guaranteed"] = guaranteed

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        price = d.pop("price", UNSET)

        distance = d.pop("distance", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, StopLossDetailsTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_stop_loss_details_time_in_force(_time_in_force)

        gtd_time = d.pop("gtdTime", UNSET)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        guaranteed = d.pop("guaranteed", UNSET)

        stop_loss_details = cls(
            price=price,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            client_extensions=client_extensions,
            guaranteed=guaranteed,
        )

        stop_loss_details.additional_properties = d
        return stop_loss_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.position import Position
    from ..models.transaction import Transaction
    from ..models.trade_summary import TradeSummary
    from ..models.order import Order


T = TypeVar("T", bound="AccountChanges")


@_attrs_define
class AccountChanges:
    """An AccountChanges Object is used to represent the changes to an Account's Orders, Trades and Positions since a
    specified Account TransactionID in the past.

        Attributes:
            orders_created (Union[Unset, List['Order']]): The Orders created. These Orders may have been filled, cancelled
                or triggered in the same period.
            orders_cancelled (Union[Unset, List['Order']]): The Orders cancelled.
            orders_filled (Union[Unset, List['Order']]): The Orders filled.
            orders_triggered (Union[Unset, List['Order']]): The Orders triggered.
            trades_opened (Union[Unset, List['TradeSummary']]): The Trades opened.
            trades_reduced (Union[Unset, List['TradeSummary']]): The Trades reduced.
            trades_closed (Union[Unset, List['TradeSummary']]): The Trades closed.
            positions (Union[Unset, List['Position']]): The Positions changed.
            transactions (Union[Unset, List['Transaction']]): The Transactions that have been generated.
    """

    orders_created: Union[Unset, List["Order"]] = UNSET
    orders_cancelled: Union[Unset, List["Order"]] = UNSET
    orders_filled: Union[Unset, List["Order"]] = UNSET
    orders_triggered: Union[Unset, List["Order"]] = UNSET
    trades_opened: Union[Unset, List["TradeSummary"]] = UNSET
    trades_reduced: Union[Unset, List["TradeSummary"]] = UNSET
    trades_closed: Union[Unset, List["TradeSummary"]] = UNSET
    positions: Union[Unset, List["Position"]] = UNSET
    transactions: Union[Unset, List["Transaction"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        orders_created: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.orders_created, Unset):
            orders_created = []
            for orders_created_item_data in self.orders_created:
                orders_created_item = orders_created_item_data.to_dict()
                orders_created.append(orders_created_item)

        orders_cancelled: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.orders_cancelled, Unset):
            orders_cancelled = []
            for orders_cancelled_item_data in self.orders_cancelled:
                orders_cancelled_item = orders_cancelled_item_data.to_dict()
                orders_cancelled.append(orders_cancelled_item)

        orders_filled: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.orders_filled, Unset):
            orders_filled = []
            for orders_filled_item_data in self.orders_filled:
                orders_filled_item = orders_filled_item_data.to_dict()
                orders_filled.append(orders_filled_item)

        orders_triggered: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.orders_triggered, Unset):
            orders_triggered = []
            for orders_triggered_item_data in self.orders_triggered:
                orders_triggered_item = orders_triggered_item_data.to_dict()
                orders_triggered.append(orders_triggered_item)

        trades_opened: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trades_opened, Unset):
            trades_opened = []
            for trades_opened_item_data in self.trades_opened:
                trades_opened_item = trades_opened_item_data.to_dict()
                trades_opened.append(trades_opened_item)

        trades_reduced: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trades_reduced, Unset):
            trades_reduced = []
            for trades_reduced_item_data in self.trades_reduced:
                trades_reduced_item = trades_reduced_item_data.to_dict()
                trades_reduced.append(trades_reduced_item)

        trades_closed: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trades_closed, Unset):
            trades_closed = []
            for trades_closed_item_data in self.trades_closed:
                trades_closed_item = trades_closed_item_data.to_dict()
                trades_closed.append(trades_closed_item)

        positions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.positions, Unset):
            positions = []
            for positions_item_data in self.positions:
                positions_item = positions_item_data.to_dict()
                positions.append(positions_item)

        transactions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.transactions, Unset):
            transactions = []
            for transactions_item_data in self.transactions:
                transactions_item = transactions_item_data.to_dict()
                transactions.append(transactions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if orders_created is not UNSET:
            field_dict["ordersCreated"] = orders_created
        if orders_cancelled is not UNSET:
            field_dict["ordersCancelled"] = orders_cancelled
        if orders_filled is not UNSET:
            field_dict["ordersFilled"] = orders_filled
        if orders_triggered is not UNSET:
            field_dict["ordersTriggered"] = orders_triggered
        if trades_opened is not UNSET:
            field_dict["tradesOpened"] = trades_opened
        if trades_reduced is not UNSET:
            field_dict["tradesReduced"] = trades_reduced
        if trades_closed is not UNSET:
            field_dict["tradesClosed"] = trades_closed
        if positions is not UNSET:
            field_dict["positions"] = positions
        if transactions is not UNSET:
            field_dict["transactions"] = transactions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.position import Position
        from ..models.transaction import Transaction
        from ..models.trade_summary import TradeSummary
        from ..models.order import Order

        d = src_dict.copy()
        orders_created = []
        _orders_created = d.pop("ordersCreated", UNSET)
        for orders_created_item_data in _orders_created or []:
            orders_created_item = Order.from_dict(orders_created_item_data)

            orders_created.append(orders_created_item)

        orders_cancelled = []
        _orders_cancelled = d.pop("ordersCancelled", UNSET)
        for orders_cancelled_item_data in _orders_cancelled or []:
            orders_cancelled_item = Order.from_dict(orders_cancelled_item_data)

            orders_cancelled.append(orders_cancelled_item)

        orders_filled = []
        _orders_filled = d.pop("ordersFilled", UNSET)
        for orders_filled_item_data in _orders_filled or []:
            orders_filled_item = Order.from_dict(orders_filled_item_data)

            orders_filled.append(orders_filled_item)

        orders_triggered = []
        _orders_triggered = d.pop("ordersTriggered", UNSET)
        for orders_triggered_item_data in _orders_triggered or []:
            orders_triggered_item = Order.from_dict(orders_triggered_item_data)

            orders_triggered.append(orders_triggered_item)

        trades_opened = []
        _trades_opened = d.pop("tradesOpened", UNSET)
        for trades_opened_item_data in _trades_opened or []:
            trades_opened_item = TradeSummary.from_dict(trades_opened_item_data)

            trades_opened.append(trades_opened_item)

        trades_reduced = []
        _trades_reduced = d.pop("tradesReduced", UNSET)
        for trades_reduced_item_data in _trades_reduced or []:
            trades_reduced_item = TradeSummary.from_dict(trades_reduced_item_data)

            trades_reduced.append(trades_reduced_item)

        trades_closed = []
        _trades_closed = d.pop("tradesClosed", UNSET)
        for trades_closed_item_data in _trades_closed or []:
            trades_closed_item = TradeSummary.from_dict(trades_closed_item_data)

            trades_closed.append(trades_closed_item)

        positions = []
        _positions = d.pop("positions", UNSET)
        for positions_item_data in _positions or []:
            positions_item = Position.from_dict(positions_item_data)

            positions.append(positions_item)

        transactions = []
        _transactions = d.pop("transactions", UNSET)
        for transactions_item_data in _transactions or []:
            transactions_item = Transaction.from_dict(transactions_item_data)

            transactions.append(transactions_item)

        account_changes = cls(
            orders_created=orders_created,
            orders_cancelled=orders_cancelled,
            orders_filled=orders_filled,
            orders_triggered=orders_triggered,
            trades_opened=trades_opened,
            trades_reduced=trades_reduced,
            trades_closed=trades_closed,
            positions=positions,
            transactions=transactions,
        )

        account_changes.additional_properties = d
        return account_changes

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
